/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.devices;

import java.util.function.Predicate;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3i;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.api.util.IBellowsConsumerBlock;
import net.dries007.tfc.client.TFCGuiHandler;
import net.dries007.tfc.objects.blocks.BlockFireBrick;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.blocks.metal.BlockMetalSheet;
import net.dries007.tfc.objects.blocks.property.ILightableBlock;
import net.dries007.tfc.objects.items.ItemFireStarter;
import net.dries007.tfc.objects.te.TEBellows;
import net.dries007.tfc.objects.te.TEBlastFurnace;
import net.dries007.tfc.objects.te.TEMetalSheet;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.Multiblock;

@ParametersAreNonnullByDefault
public class BlockBlastFurnace extends Block implements IBellowsConsumerBlock, ILightableBlock
{
    private static final Multiblock BLAST_FURNACE_CHIMNEY;

    static
    {
        Predicate<IBlockState> stoneMatcher = state -> state.func_177230_c() instanceof BlockFireBrick;
        Predicate<IBlockState> ironSheetMatcher = state -> (state.func_177230_c() instanceof BlockMetalSheet)
            && ((BlockMetalSheet) state.func_177230_c()).getMetal() == Metal.WROUGHT_IRON;
        BLAST_FURNACE_CHIMNEY = new Multiblock()
            .match(new BlockPos(0, 0, 0), state -> state.func_177230_c() == BlocksTFC.MOLTEN || state.func_177230_c() == Blocks.field_150350_a)
            .match(new BlockPos(0, 0, 1), stoneMatcher)
            .match(new BlockPos(0, 0, -1), stoneMatcher)
            .match(new BlockPos(1, 0, 0), stoneMatcher)
            .match(new BlockPos(-1, 0, 0), stoneMatcher)
            .match(new BlockPos(0, 0, -2), ironSheetMatcher)
            .match(new BlockPos(0, 0, -2), tile -> tile.getFace(EnumFacing.NORTH), TEMetalSheet.class)
            .match(new BlockPos(0, 0, 2), ironSheetMatcher)
            .match(new BlockPos(0, 0, 2), tile -> tile.getFace(EnumFacing.SOUTH), TEMetalSheet.class)
            .match(new BlockPos(2, 0, 0), ironSheetMatcher)
            .match(new BlockPos(2, 0, 0), tile -> tile.getFace(EnumFacing.EAST), TEMetalSheet.class)
            .match(new BlockPos(-2, 0, 0), ironSheetMatcher)
            .match(new BlockPos(-2, 0, 0), tile -> tile.getFace(EnumFacing.WEST), TEMetalSheet.class)
            .match(new BlockPos(-1, 0, -1), ironSheetMatcher)
            .match(new BlockPos(-1, 0, -1), tile -> tile.getFace(EnumFacing.NORTH) && tile.getFace(EnumFacing.WEST), TEMetalSheet.class)
            .match(new BlockPos(1, 0, -1), ironSheetMatcher)
            .match(new BlockPos(1, 0, -1), tile -> tile.getFace(EnumFacing.NORTH) && tile.getFace(EnumFacing.EAST), TEMetalSheet.class)
            .match(new BlockPos(-1, 0, 1), ironSheetMatcher)
            .match(new BlockPos(-1, 0, 1), tile -> tile.getFace(EnumFacing.SOUTH) && tile.getFace(EnumFacing.WEST), TEMetalSheet.class)
            .match(new BlockPos(1, 0, 1), ironSheetMatcher)
            .match(new BlockPos(1, 0, 1), tile -> tile.getFace(EnumFacing.SOUTH) && tile.getFace(EnumFacing.EAST), TEMetalSheet.class);
    }

    public BlockBlastFurnace()
    {
        super(Material.field_151573_f);
    }

    public int getChimneyLevels(World world, BlockPos pos)
    {
        if (world.func_180495_p(pos.func_177977_b()).func_177230_c() != BlocksTFC.CRUCIBLE)
        {
            // no crucible
            return 0;
        }
        for (int i = 1; i < 6; i++)
        {
            BlockPos center = pos.func_177981_b(i);
            if (!BLAST_FURNACE_CHIMNEY.test(world, center))
            {
                return i - 1;
            }
        }
        // Maximum levels
        return 5;
    }

    @SuppressWarnings("deprecation")
    @Nonnull
    @Override
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P().func_177226_a(LIT, meta == 1);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(LIT) ? 1 : 0;
    }


    @Override
    public void func_180663_b(World worldIn, BlockPos pos, IBlockState state)
    {
        TEBlastFurnace te = Helpers.getTE(worldIn, pos, TEBlastFurnace.class);
        if (te != null)
        {
            te.onBreakBlock(worldIn, pos);
        }
        super.func_180663_b(worldIn, pos, state);
    }

    @Override
    public boolean func_176196_c(World worldIn, @Nonnull BlockPos pos)
    {
        if (!super.func_176196_c(worldIn, pos))
            return false;

        return getChimneyLevels(worldIn, pos) > 0;
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        if (!worldIn.field_72995_K)
        {
            if (!state.func_177229_b(LIT))
            {
                TEBlastFurnace te = Helpers.getTE(worldIn, pos, TEBlastFurnace.class);
                if (te == null)
                    return true;
                ItemStack held = playerIn.func_184586_b(hand);
                if (ItemFireStarter.canIgnite(held) && te.canIgnite())
                {
                    worldIn.func_175656_a(pos, state.func_177226_a(LIT, true));
                    //te.onIgnite();
                    return true;
                }
            }
            if (!playerIn.func_70093_af())
            {
                TFCGuiHandler.openGui(worldIn, pos, playerIn, TFCGuiHandler.Type.BLAST_FURNACE);
            }
        }
        return true;
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, LIT);
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TEBlastFurnace();
    }

    @Override
    public boolean canIntakeFrom(@Nonnull TEBellows te, @Nonnull Vec3i offset, @Nonnull EnumFacing facing)
    {
        return offset.equals(TEBellows.OFFSET_LEVEL);
    }

    @Override
    public void onAirIntake(@Nonnull TEBellows te, @Nonnull World world, @Nonnull BlockPos pos, int airAmount)
    {
        TEBlastFurnace teBlastFurnace = Helpers.getTE(world, pos, TEBlastFurnace.class);
        if (teBlastFurnace != null)
        {
            teBlastFurnace.onAirIntake(airAmount);
        }
    }
}
