/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.devices;

import java.util.Random;
import java.util.function.BiPredicate;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.DamageSource;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.math.Vec3i;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.api.util.IBellowsConsumerBlock;
import net.dries007.tfc.client.TFCGuiHandler;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.blocks.property.ILightableBlock;
import net.dries007.tfc.objects.items.ItemFireStarter;
import net.dries007.tfc.objects.te.TEBellows;
import net.dries007.tfc.objects.te.TECharcoalForge;
import net.dries007.tfc.objects.te.TEInventory;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.Multiblock;

@ParametersAreNonnullByDefault
public class BlockCharcoalForge extends Block implements IBellowsConsumerBlock, ILightableBlock
{
    private static final AxisAlignedBB AABB = new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 0.875D, 1.0D);
    private static final Multiblock CHARCOAL_FORGE_MULTIBLOCK;

    static
    {
        BiPredicate<World, BlockPos> skyMatcher = World::func_175710_j;
        BiPredicate<World, BlockPos> stoneMatcher = (world, pos) ->
        {
            IBlockState state = world.func_180495_p(pos);
            return state.func_185904_a() == Material.field_151576_e && state.func_185914_p() && state.func_185915_l();
        };
        CHARCOAL_FORGE_MULTIBLOCK = new Multiblock()
            // Top block
            .match(new BlockPos(0, 1, 0), state -> state.func_177230_c() == BlocksTFC.CRUCIBLE || state.func_177230_c() == Blocks.field_150350_a)
            // Chimney
            .matchOneOf(new BlockPos(0, 1, 0), new Multiblock()
                .match(new BlockPos(0, 0, 0), skyMatcher)
                .match(new BlockPos(0, 0, 1), skyMatcher)
                .match(new BlockPos(0, 0, 2), skyMatcher)
                .match(new BlockPos(0, 0, -1), skyMatcher)
                .match(new BlockPos(0, 0, -2), skyMatcher)
                .match(new BlockPos(1, 0, 0), skyMatcher)
                .match(new BlockPos(2, 0, 0), skyMatcher)
                .match(new BlockPos(-1, 0, 0), skyMatcher)
                .match(new BlockPos(-2, 0, 0), skyMatcher)
            )
            // Underneath
            .match(new BlockPos(1, 0, 0), stoneMatcher)
            .match(new BlockPos(-1, 0, 0), stoneMatcher)
            .match(new BlockPos(0, 0, 1), stoneMatcher)
            .match(new BlockPos(0, 0, -1), stoneMatcher)
            .match(new BlockPos(0, -1, 0), stoneMatcher);
    }

    public static boolean isValid(World world, BlockPos pos)
    {
        return CHARCOAL_FORGE_MULTIBLOCK.test(world, pos);
    }

    public static boolean hasValidChimney(World world, BlockPos pos)
    {
        return CHARCOAL_FORGE_MULTIBLOCK.test(world, pos);
    }

    public BlockCharcoalForge()
    {
        super(Material.field_151578_c);

        func_149672_a(SoundType.field_185849_b);
        setHarvestLevel("shovel", 0);
        func_149711_c(1.0F);
        func_149675_a(true); // Used for chimney checks -> extinguish
        this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(LIT, false));
    }

    @Override
    public boolean canIntakeFrom(TEBellows te, Vec3i offset, EnumFacing facing)
    {
        return offset.equals(TEBellows.OFFSET_INSET);
    }

    @Override
    public void onAirIntake(TEBellows te, World world, BlockPos pos, int airAmount)
    {
        TECharcoalForge teForge = Helpers.getTE(world, pos, TECharcoalForge.class);
        if (teForge != null)
        {
            teForge.onAirIntake(airAmount);
        }
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_185481_k(IBlockState state)
    {
        return false;
    }

    @Nonnull
    @SuppressWarnings("deprecation")
    @Override
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P().func_177226_a(LIT, meta == 1);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(LIT) ? 1 : 0;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return AABB;
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return face.func_176740_k() == EnumFacing.Axis.Y ? BlockFaceShape.SOLID : BlockFaceShape.UNDEFINED;
    }

    @Override
    @Nullable
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_180646_a(IBlockState state, IBlockAccess worldIn, BlockPos pos)
    {
        return AABB;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        if (!isValid(worldIn, pos))
        {
            worldIn.func_175656_a(pos, state.func_177226_a(LIT, false));
        }
    }

    @Override
    @SuppressWarnings("deprecation")
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        if (!worldIn.field_72995_K)
        {
            if (state.func_177229_b(LIT) && !isValid(worldIn, pos))
            {
                // This is not a valid pit, therefor extinguish it
                worldIn.func_175656_a(pos, state.func_177226_a(LIT, false));
            }
        }
    }

    @Override
    public int func_149745_a(Random random)
    {
        return 7;
    }

    @Override
    @Nonnull
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        return Items.field_151044_h;
    }

    @Override
    public int func_180651_a(IBlockState state)
    {
        return 1;
    }

    @Override
    public boolean func_180639_a(World world, BlockPos pos, IBlockState state, EntityPlayer player, EnumHand hand, EnumFacing side, float hitX, float hitY, float hitZ)
    {
        if (!world.field_72995_K)
        {
            if (!state.func_177229_b(LIT))
            {
                ItemStack held = player.func_184586_b(hand);
                if (ItemFireStarter.canIgnite(held) && isValid(world, pos))
                {
                    world.func_175656_a(pos, state.func_177226_a(LIT, true));
                    return true;
                }
            }
            if (!player.func_70093_af())
            {
                TFCGuiHandler.openGui(world, pos, player, TFCGuiHandler.Type.CHARCOAL_FORGE);
            }
        }
        return true;
    }

    @Override
    public void func_176199_a(World worldIn, BlockPos pos, Entity entityIn)
    {
        IBlockState state = worldIn.func_180495_p(pos);
        if (state.func_177229_b(LIT) && !entityIn.func_70045_F() && entityIn instanceof EntityLivingBase && state.func_177229_b(LIT))
        {
            entityIn.func_70097_a(DamageSource.field_76372_a, 2.0F);
        }
        super.func_176199_a(worldIn, pos, entityIn);
    }

    @Override
    public void func_180657_a(World worldIn, EntityPlayer player, BlockPos pos, IBlockState state, @Nullable TileEntity te, ItemStack stack)
    {
        if (!worldIn.field_72995_K && te instanceof TEInventory)
        {
            ((TEInventory) te).onBreakBlock(worldIn, pos);
        }
        super.func_180657_a(worldIn, player, pos, state, te, stack);
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, LIT);
    }

    @Override
    public int getLightValue(IBlockState state, IBlockAccess world, BlockPos pos)
    {
        return state.func_177229_b(LIT) ? 15 : 0;
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TECharcoalForge();
    }

    @Override
    @Nonnull
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        return new ItemStack(Items.field_151044_h, 1, 1);
    }

}
