/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.entity.animal;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.*;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.DamageSource;
import net.minecraft.util.EnumHand;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.TextComponentTranslation;
import net.minecraft.world.World;
import net.minecraft.world.biome.Biome;
import net.minecraftforge.oredict.OreDictionary;

import net.dries007.tfc.Constants;
import net.dries007.tfc.objects.LootTablesTFC;
import net.dries007.tfc.util.calendar.CalendarTFC;
import net.dries007.tfc.world.classic.biomes.BiomesTFC;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Age;
import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Gender;

@ParametersAreNonnullByDefault
public class EntityCowTFC extends EntityAnimalMammal implements IAnimalTFC
{
    private static final int DAYS_TO_ADULTHOOD = 1080;
    private static final int DAYS_TO_FULL_GESTATION = 270;

    private static int getRandomGrowth()
    {
        int lifeTimeDays = Constants.RNG.nextInt(DAYS_TO_ADULTHOOD * 4);
        return (int) (CalendarTFC.PLAYER_TIME.getTotalDays() - lifeTimeDays);
    }
    private long lastMilked;

    @SuppressWarnings("unused")
    public EntityCowTFC(World worldIn)
    {
        this(worldIn, Gender.fromBool(Constants.RNG.nextBoolean()),
            getRandomGrowth());
    }

    public EntityCowTFC(World worldIn, Gender gender, int birthDay)
    {
        super(worldIn, gender, birthDay);
        this.func_70105_a(0.9F, 1.3F);
        this.setMilkedDay(-1); //Spawn with milk
    }

    @Override
    public boolean isValidSpawnConditions(Biome biome, float temperature, float rainfall)
    {
        return (biome == BiomesTFC.PLAINS || biome == BiomesTFC.HIGH_PLAINS) && temperature > -10 && rainfall > 100 && rainfall < 400;
    }

    @Override
    public void func_70636_d()
    {
        super.func_70636_d();
        if (!this.field_70170_p.field_72995_K)
        {
            if (this.getMilkedDay() > CalendarTFC.PLAYER_TIME.getTotalDays())
            {
                //Calendar went backwards by command! this need to update
                this.setMilkedDay((int) CalendarTFC.PLAYER_TIME.getTotalDays());
            }
        }
    }

    @Override
    public void func_70014_b(NBTTagCompound compound)
    {
        super.func_70014_b(compound);
        compound.func_74772_a("milked", this.getMilkedDay());
    }

    @Override
    public void func_70037_a(NBTTagCompound compound)
    {
        super.func_70037_a(compound);
        this.setMilkedDay(compound.func_74762_e("milked"));
    }

    @Override
    public void birthChildren()
    {
        int numberOfChilds = 1; //one always
        for (int i = 0; i < numberOfChilds; i++)
        {
            EntityCowTFC baby = new EntityCowTFC(this.field_70170_p, Gender.fromBool(Constants.RNG.nextBoolean()), (int) CalendarTFC.PLAYER_TIME.getTotalDays());
            baby.func_70012_b(this.field_70165_t, this.field_70163_u, this.field_70161_v, 0.0F, 0.0F);
            this.field_70170_p.func_72838_d(baby);
        }

    }

    @Override
    public long gestationDays()
    {
        return DAYS_TO_FULL_GESTATION;
    }

    @Override
    public boolean func_184645_a(EntityPlayer player, EnumHand hand)
    {
        ItemStack itemstack = player.func_184586_b(hand);

        if (itemstack.func_77973_b() == Items.field_151133_ar)
        {
            if (this.getFamiliarity() > 0.15f && hasMilk())
            {
                player.func_184185_a(SoundEvents.field_187564_an, 1.0F, 1.0F);
                this.setMilkedDay(CalendarTFC.PLAYER_TIME.getTotalDays());
                itemstack.func_190918_g(1);

                if (itemstack.func_190926_b())
                {
                    player.func_184611_a(hand, new ItemStack(Items.field_151117_aB));
                }
                else if (!player.field_71071_by.func_70441_a(new ItemStack(Items.field_151117_aB)))
                {
                    player.func_71019_a(new ItemStack(Items.field_151117_aB), false);
                }
            }
            else if (!this.field_70170_p.field_72995_K)
            {
                //Return chat message indicating why this entity isn't giving milk
                if (this.getGender() == Gender.MALE)
                {
                    player.func_145747_a(new TextComponentTranslation(MOD_ID + ".tooltip.animal.milk.male"));
                }
                else if (this.getAge() == Age.OLD)
                {
                    player.func_145747_a(new TextComponentTranslation(MOD_ID + ".tooltip.animal.milk.old"));
                }
                else if (this.getAge() == Age.CHILD)
                {
                    player.func_145747_a(new TextComponentTranslation(MOD_ID + ".tooltip.animal.milk.child"));
                }
                else if (getFamiliarity() <= 0.15f)
                {
                    player.func_145747_a(new TextComponentTranslation(MOD_ID + ".tooltip.animal.milk.lowfamiliarity"));
                }
                else if (!hasMilk())
                {
                    player.func_145747_a(new TextComponentTranslation(MOD_ID + ".tooltip.animal.milk.empty"));
                }
            }
            return true;
        }
        else
        {
            return super.func_184645_a(player, hand);
        }
    }

    @Override
    public float getPercentToAdulthood()
    {
        if (this.getAge() != Age.CHILD) return 1;
        double value = (CalendarTFC.PLAYER_TIME.getTotalDays() - this.getBirthDay()) / (double) DAYS_TO_ADULTHOOD;
        if (value > 1f) value = 1f;
        if (value < 0f) value = 0;
        return (float) value;
    }

    @Override
    public Age getAge()
    {
        return CalendarTFC.PLAYER_TIME.getTotalDays() >= this.getBirthDay() + DAYS_TO_ADULTHOOD ? Age.ADULT : Age.CHILD;
    }

    public long getMilkedDay()
    {
        return this.lastMilked;
    }

    public void setMilkedDay(long value)
    {
        this.lastMilked = value;
    }

    public boolean hasMilk()
    {
        return this.getGender() == Gender.FEMALE && this.getAge() == Age.ADULT && (this.getMilkedDay() == -1 || CalendarTFC.PLAYER_TIME.getTotalDays() > getMilkedDay());
    }

    @Override
    protected SoundEvent func_184601_bQ(DamageSource damageSourceIn)
    {
        return SoundEvents.field_187562_am;
    }

    @Override
    protected SoundEvent func_184615_bR()
    {
        return SoundEvents.field_187560_al;
    }

    @Override
    protected void func_184651_r()
    {
        this.field_70714_bg.func_75776_a(0, new EntityAISwimming(this));
        this.field_70714_bg.func_75776_a(1, new EntityAIPanic(this, 1.3D));
        this.field_70714_bg.func_75776_a(2, new EntityAIMate(this, 1.0D));
        for (ItemStack is : OreDictionary.getOres("grain"))
        {
            Item item = is.func_77973_b();
            this.field_70714_bg.func_75776_a(3, new EntityAITempt(this, 1.1D, item, false));
        }
        this.field_70714_bg.func_75776_a(4, new EntityAIFollowParent(this, 1.1D));
        this.field_70714_bg.func_75776_a(5, new EntityAIWanderAvoidWater(this, 1.0D));
        this.field_70714_bg.func_75776_a(6, new EntityAIWatchClosest(this, EntityPlayer.class, 6.0F));
        this.field_70714_bg.func_75776_a(7, new EntityAILookIdle(this));
    }

    @Override
    protected void func_110147_ax()
    {
        super.func_110147_ax();
        this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(10.0D);
        this.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.20000000298023224D);
    }

    @Override
    protected SoundEvent func_184639_G()
    {
        return SoundEvents.field_187558_ak;
    }

    @Nullable
    protected ResourceLocation func_184647_J()
    {
        return LootTablesTFC.ANIMALS_COW;
    }

    @Override
    protected void func_180429_a(BlockPos pos, Block blockIn)
    {
        this.func_184185_a(SoundEvents.field_187566_ao, 0.15F, 1.0F);
    }
}
