/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.entity.animal;

import java.util.List;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.*;
import net.minecraft.entity.passive.EntitySheep;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.util.DamageSource;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraft.world.biome.Biome;
import net.minecraftforge.common.IShearable;
import net.minecraftforge.oredict.OreDictionary;

import net.dries007.tfc.Constants;
import net.dries007.tfc.objects.LootTablesTFC;
import net.dries007.tfc.objects.items.ItemsTFC;
import net.dries007.tfc.util.calendar.CalendarTFC;
import net.dries007.tfc.world.classic.biomes.BiomesTFC;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Age;
import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Gender;

@ParametersAreNonnullByDefault
public class EntitySheepTFC extends EntityAnimalMammal implements IShearable, IAnimalTFC
{
    private static final int DAYS_TO_ADULTHOOD = 360;
    private static final int DAYS_TO_GROW_WOOL = 7;
    private static final int DAYS_TO_FULL_GESTATION = 150;

    private static final DataParameter<Integer> DYE_COLOR = EntityDataManager.func_187226_a(EntitySheepTFC.class, DataSerializers.field_187192_b);
    private static final DataParameter<Integer> SHEARED = EntityDataManager.func_187226_a(EntitySheepTFC.class, DataSerializers.field_187192_b);

    private static int getRandomGrowth()
    {
        // Used when natural spawning sheeps
        int lifeTimeDays = Constants.RNG.nextInt(DAYS_TO_ADULTHOOD * 4); // 3 out of 4 natural spawned sheeps will be adults
        return (int) (CalendarTFC.PLAYER_TIME.getTotalDays() - lifeTimeDays);
    }

    public EntitySheepTFC(World worldIn)
    {
        this(worldIn, Gender.fromBool(Constants.RNG.nextBoolean()),
            getRandomGrowth(),
            EntitySheep.func_175510_a(Constants.RNG));
    }

    public EntitySheepTFC(World worldIn, Gender gender, int birthDay, EnumDyeColor dye)
    {
        super(worldIn, gender, birthDay);
        this.func_70105_a(0.9F, 1.3F);
        this.setDyeColor(dye);
        this.setShearedDay(-1); //Spawn with wool
    }

    @Override
    public boolean isValidSpawnConditions(Biome biome, float temperature, float rainfall)
    {
        return (temperature > -20 && temperature < 0 && rainfall > 100) ||
            (temperature > -10 && rainfall > 100 && biome == BiomesTFC.MOUNTAINS);
    }

    @Override
    public void func_70636_d()
    {
        super.func_70636_d();
        if (!this.field_70170_p.field_72995_K)
        {
            if (this.getShearedDay() > CalendarTFC.PLAYER_TIME.getTotalDays())
            {
                //Calendar went backwards by command! this need to update
                this.setShearedDay((int) CalendarTFC.PLAYER_TIME.getTotalDays());
            }
        }
    }

    @Override
    public void func_70014_b(NBTTagCompound compound)
    {
        super.func_70014_b(compound);
        compound.func_74768_a("sheared", this.getShearedDay());
        compound.func_74768_a("dyecolor", this.getDyeColor().func_176765_a());
    }

    @Override
    public void func_70037_a(NBTTagCompound compound)
    {
        super.func_70037_a(compound);
        this.setShearedDay(compound.func_74762_e("sheared"));
        this.setDyeColor(EnumDyeColor.func_176764_b(compound.func_74771_c("dyecolor")));
    }

    @Override
    public void birthChildren()
    {
        int numberOfChilds = Constants.RNG.nextInt(3) + 1; //1-3
        for (int i = 0; i < numberOfChilds; i++)
        {
            EntitySheepTFC baby = new EntitySheepTFC(this.field_70170_p, Gender.fromBool(Constants.RNG.nextBoolean()), (int) CalendarTFC.PLAYER_TIME.getTotalDays(), this.getDyeColor());
            baby.func_70012_b(this.field_70165_t, this.field_70163_u, this.field_70161_v, 0.0F, 0.0F);
            this.field_70170_p.func_72838_d(baby);
        }

    }

    @Override
    public long gestationDays()
    {
        return DAYS_TO_FULL_GESTATION;
    }

    public EnumDyeColor getDyeColor()
    {
        return EnumDyeColor.func_176764_b(this.field_70180_af.func_187225_a(DYE_COLOR));
    }

    public void setDyeColor(EnumDyeColor color)
    {
        this.field_70180_af.func_187227_b(DYE_COLOR, color.func_176765_a());
    }

    @Override
    public boolean isShearable(@Nonnull ItemStack item, IBlockAccess world, BlockPos pos)
    {
        return getAge() == Age.ADULT && hasWool();
    }

    @Nonnull
    @Override
    public List<ItemStack> onSheared(@Nonnull ItemStack item, IBlockAccess world, BlockPos pos, int fortune)
    {
        this.setShearedDay((int) CalendarTFC.PLAYER_TIME.getTotalDays());
        int i = 1 + this.field_70146_Z.nextInt(3);

        java.util.List<ItemStack> ret = new java.util.ArrayList<>();
        for (int j = 0; j < i; ++j)
            ret.add(new ItemStack(ItemsTFC.WOOL, 1, this.getDyeColor().func_176765_a()));

        this.func_184185_a(SoundEvents.field_187763_eJ, 1.0F, 1.0F);
        return ret;
    }

    public int getShearedDay()
    {
        return this.field_70180_af.func_187225_a(SHEARED);
    }

    public void setShearedDay(int value)
    {
        this.field_70180_af.func_187227_b(SHEARED, value);
    }

    public boolean hasWool()
    {
        return this.getShearedDay() == -1 || CalendarTFC.PLAYER_TIME.getTotalDays() >= getShearedDay() + DAYS_TO_GROW_WOOL;
    }

    @Override
    protected void func_70088_a()
    {
        super.func_70088_a();
        this.field_70180_af.func_187214_a(DYE_COLOR, 0);
        this.field_70180_af.func_187214_a(SHEARED, 0);
    }

    @Override
    public float getPercentToAdulthood()
    {
        if (this.getAge() != Age.CHILD) return 1;
        double value = (CalendarTFC.PLAYER_TIME.getTotalDays() - this.getBirthDay()) / (double) DAYS_TO_ADULTHOOD;
        if (value > 1f) value = 1f;
        if (value < 0f) value = 0;
        return (float) value;
    }

    @Override
    public Age getAge()
    {
        return CalendarTFC.PLAYER_TIME.getTotalDays() >= this.getBirthDay() + DAYS_TO_ADULTHOOD ? Age.ADULT : Age.CHILD;
    }

    @Override
    protected SoundEvent func_184601_bQ(DamageSource damageSourceIn)
    {
        return SoundEvents.field_187761_eI;
    }

    @Override
    protected SoundEvent func_184615_bR()
    {
        return SoundEvents.field_187759_eH;
    }

    @Override
    protected void func_184651_r()
    {
        this.field_70714_bg.func_75776_a(0, new EntityAISwimming(this));
        this.field_70714_bg.func_75776_a(1, new EntityAIPanic(this, 1.3D));
        this.field_70714_bg.func_75776_a(2, new EntityAIMate(this, 1.0D));
        for (ItemStack is : OreDictionary.getOres("grain"))
        {
            Item item = is.func_77973_b();
            this.field_70714_bg.func_75776_a(3, new EntityAITempt(this, 1.1D, item, false));
        }
        this.field_70714_bg.func_75776_a(4, new EntityAIFollowParent(this, 1.1D));
        this.field_70714_bg.func_75776_a(5, new EntityAIWanderAvoidWater(this, 1.0D));
        this.field_70714_bg.func_75776_a(6, new EntityAIWatchClosest(this, EntityPlayer.class, 6.0F));
        this.field_70714_bg.func_75776_a(7, new EntityAILookIdle(this));
    }

    @Override
    protected void func_110147_ax()
    {
        super.func_110147_ax();
        this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(8.0D);
        this.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.23D);
    }

    @Override
    protected SoundEvent func_184639_G()
    {
        return SoundEvents.field_187757_eG;
    }

    @Nullable
    protected ResourceLocation func_184647_J()
    {
        return LootTablesTFC.ANIMALS_SHEEP;
    }

    @Override
    protected void func_180429_a(BlockPos pos, Block blockIn)
    {
        this.func_184185_a(SoundEvents.field_187765_eK, 0.15F, 1.0F);
    }
}
