/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.recipes;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import com.google.gson.JsonObject;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.util.JsonUtils;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.World;
import net.minecraftforge.common.ForgeHooks;
import net.minecraftforge.common.crafting.IRecipeFactory;
import net.minecraftforge.common.crafting.JsonContext;
import net.minecraftforge.fluids.capability.IFluidHandler;
import net.minecraftforge.items.ItemHandlerHelper;
import net.minecraftforge.oredict.ShapelessOreRecipe;

import net.dries007.tfc.Constants;
import net.dries007.tfc.api.capability.IMoldHandler;
import net.dries007.tfc.api.capability.heat.IItemHeat;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.objects.items.ceramics.ItemMold;
import net.dries007.tfc.objects.items.metal.ItemMetal;

import static net.dries007.tfc.api.capability.heat.CapabilityItemHeat.ITEM_HEAT_CAPABILITY;
import static net.minecraftforge.fluids.capability.CapabilityFluidHandler.FLUID_HANDLER_CAPABILITY;

@SuppressWarnings("unused")
@ParametersAreNonnullByDefault
public class UnmoldRecipe extends ShapelessOreRecipe
{
    private Metal.ItemType type;
    private float chance;

    private UnmoldRecipe(ResourceLocation group, NonNullList<Ingredient> input, @Nonnull Metal.ItemType type, float chance)
    {
        super(group, input, ItemStack.field_190927_a);
        this.type = type;
        this.chance = chance;
    }

    @Override
    @Nonnull
    public NonNullList<ItemStack> func_179532_b(final InventoryCrafting inv)
    {
        // Return empty molds
        for (int slot = 0; slot < inv.func_70302_i_(); slot++)
        {
            ItemStack stack = inv.func_70301_a(slot);
            if (!stack.func_190926_b())
            {
                if (stack.func_77973_b() instanceof ItemMold)
                {
                    // No need to check for the mold, as it has already been checked earlier
                    if (Constants.RNG.nextFloat() <= chance)
                    {
                        EntityPlayer player = ForgeHooks.getCraftingPlayer();
                        if (!player.field_70170_p.field_72995_K)
                        {
                            // This can't use the remaining items, because vanilla doesn't sync them on crafting, thus it gives a desync error
                            // To fix: ContainerWorkbench#onCraftMatrixChanged needs to call Container#detectAndSendChanges
                            ItemHandlerHelper.giveItemToPlayer(player, new ItemStack(stack.func_77973_b()));
                        }
                    }
                }
            }
        }
        return super.func_179532_b(inv);
    }

    @Override
    public boolean func_192399_d()
    {
        return true;
    }

    @Override
    @Nonnull
    public ItemStack func_77571_b() { return ItemStack.field_190927_a; }

    @Override
    @Nonnull
    public ItemStack func_77572_b(InventoryCrafting inv)
    {
        ItemStack moldStack = null;
        for (int slot = 0; slot < inv.func_70302_i_(); slot++)
        {
            ItemStack stack = inv.func_70301_a(slot);
            if (!stack.func_190926_b())
            {
                if (stack.func_77973_b() instanceof ItemMold)
                {
                    ItemMold tmp = ((ItemMold) stack.func_77973_b());
                    if (tmp.type.equals(this.type) && moldStack == null)
                    {
                        moldStack = stack;
                    }
                    else
                    {
                        return ItemStack.field_190927_a;
                    }
                }
                else
                {
                    return ItemStack.field_190927_a;
                }
            }
        }
        if (moldStack != null)
        {
            IFluidHandler moldCap = moldStack.getCapability(FLUID_HANDLER_CAPABILITY, null);
            if (moldCap instanceof IMoldHandler)
            {
                IMoldHandler moldHandler = (IMoldHandler) moldCap;
                if (!moldHandler.isMolten() && moldHandler.getAmount() == 100)
                {
                    return getOutputItem(moldHandler, this.type);
                }
            }
        }
        return ItemStack.field_190927_a;
    }

    @Override
    public boolean func_77569_a(@Nonnull InventoryCrafting inv, @Nonnull World world)
    {
        boolean foundMold = false;
        for (int slot = 0; slot < inv.func_70302_i_(); slot++)
        {
            ItemStack stack = inv.func_70301_a(slot);
            if (!stack.func_190926_b())
            {
                if (stack.func_77973_b() instanceof ItemMold)
                {
                    ItemMold moldItem = ((ItemMold) stack.func_77973_b());
                    IFluidHandler cap = stack.getCapability(FLUID_HANDLER_CAPABILITY, null);

                    if (cap instanceof IMoldHandler)
                    {
                        IMoldHandler moldHandler = (IMoldHandler) cap;
                        if (!moldHandler.isMolten())
                        {
                            Metal metal = moldHandler.getMetal();
                            if (metal != null && moldItem.type.equals(this.type) && !foundMold)
                            {
                                foundMold = true;
                            }
                            else
                            {
                                return false;
                            }
                        }
                        else
                        {
                            return false;
                        }
                    }
                    else
                    {
                        return false;
                    }
                }
                else
                {
                    return false;
                }
            }
        }
        return foundMold;
    }

    @Override
    @Nonnull
    public String func_193358_e()
    {
        return group == null ? "" : group.toString();
    }

    @Override
    public boolean func_194133_a(int width, int height)
    {
        return true;
    }

    private ItemStack getOutputItem(final IMoldHandler moldHandler, final Metal.ItemType type)
    {
        Metal m = moldHandler.getMetal();
        if (m != null)
        {
            ItemStack output = new ItemStack(ItemMetal.get(m, type));
            IItemHeat heat = output.getCapability(ITEM_HEAT_CAPABILITY, null);
            if (heat != null)
            {
                heat.setTemperature(moldHandler.getTemperature());
            }
            return output;
        }
        return ItemStack.field_190927_a;
    }

    @SuppressWarnings("unused")
    public static class Factory implements IRecipeFactory
    {
        @Override
        public IRecipe parse(final JsonContext context, final JsonObject json)
        {
            final NonNullList<Ingredient> ingredients = RecipeUtils.parseShapeless(context, json);
            final String result = JsonUtils.func_151200_h(json, "result");
            final Metal.ItemType type = Metal.ItemType.valueOf(result.toUpperCase());
            final String group = JsonUtils.func_151219_a(json, "group", "");

            //Chance of getting the mold back
            float chance = 0;
            if (JsonUtils.func_151204_g(json, "chance"))
            {
                chance = JsonUtils.func_151217_k(json, "chance");
            }

            return new UnmoldRecipe(group.isEmpty() ? new ResourceLocation(result) : new ResourceLocation(group), ingredients, type, chance);
        }
    }
}
