/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.inventory.ItemStackHelper;
import net.minecraft.item.Item;
import net.minecraft.item.ItemFlintAndSteel;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;
import net.minecraftforge.items.ItemHandlerHelper;

import net.dries007.tfc.ConfigTFC;
import net.dries007.tfc.api.capability.heat.CapabilityItemHeat;
import net.dries007.tfc.api.capability.heat.IItemHeat;
import net.dries007.tfc.api.recipes.heat.HeatRecipe;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.items.ItemsTFC;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;

@ParametersAreNonnullByDefault
public class TEPitKiln extends TEPlacedItem implements ITickable
{
    public static final int STRAW_NEEDED = 8;
    public static final int WOOD_NEEDED = 8;

    public static void convertPlacedItemToPitKiln(World world, BlockPos pos, ItemStack strawStack)
    {
        TEPlacedItem teOld = Helpers.getTE(world, pos, TEPlacedItem.class);
        if (teOld != null)
        {
            // Remove inventory items
            // This happens here to stop the block dropping its items in onBreakBlock()
            IItemHandler capOld = teOld.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
            ItemStack[] inventory = new ItemStack[4];
            if (capOld != null)
            {
                for (int i = 0; i < 4; i++)
                {
                    inventory[i] = capOld.extractItem(i, 64, false);
                }
            }

            // Replace the block
            world.func_175656_a(pos, BlocksTFC.PIT_KILN.func_176223_P());
            // Copy TE data
            TEPitKiln teNew = Helpers.getTE(world, pos, TEPitKiln.class);
            if (teNew != null)
            {
                // Copy inventory
                IItemHandler capNew = teNew.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
                if (capNew != null)
                {
                    for (int i = 0; i < 4; i++)
                    {
                        if (inventory[i] != null && !inventory[i].func_190926_b())
                        {
                            capNew.insertItem(i, inventory[i], false);
                        }
                    }
                }
                // Copy misc data
                teNew.isHoldingLargeItem = teOld.isHoldingLargeItem;
                if (OreDictionaryHelper.doesStackMatchOre(strawStack, "blockStraw"))
                {
                    teNew.addStrawBlock();
                }
                else
                {
                    teNew.addStraw(strawStack.func_77979_a(1));
                }
            }
        }
    }

    private final NonNullList<ItemStack> logItems = NonNullList.func_191197_a(WOOD_NEEDED, ItemStack.field_190927_a);
    private final NonNullList<ItemStack> strawItems = NonNullList.func_191197_a(STRAW_NEEDED, ItemStack.field_190927_a);
    private int burnTicksToGo;

    @Override
    public void func_73660_a()
    {
        if (burnTicksToGo > 0)
        {
            burnTicksToGo--;
            BlockPos above = field_174879_c.func_177984_a();
            if (field_145850_b.func_175623_d(above))
            {
                field_145850_b.func_175656_a(above, Blocks.field_150480_ab.func_176223_P());
            }
            else
            {
                IBlockState stateAbove = field_145850_b.func_180495_p(above);
                if (stateAbove.func_185904_a() != Material.field_151581_o)
                {
                    // todo: decide what to do now.
                    burnTicksToGo = 0;
                    return;
                }
            }
            if (burnTicksToGo == 0)
            {
                strawItems.clear();
                logItems.clear();

                for (int i = 0; i < inventory.getSlots(); i++)
                {
                    ItemStack stack = inventory.getStackInSlot(i);
                    ItemStack outputStack = ItemStack.field_190927_a;
                    // First, heat up the item to max temperature, so the recipe can properly check the temperature of the item
                    IItemHeat heat = stack.getCapability(CapabilityItemHeat.ITEM_HEAT_CAPABILITY, null);
                    if (heat != null)
                    {
                        heat.setTemperature(CapabilityItemHeat.MAX_TEMPERATURE);

                        // Only Tier I and below can be melted in a pit kiln
                        HeatRecipe recipe = HeatRecipe.get(stack, Metal.Tier.TIER_I);
                        if (recipe != null)
                        {
                            outputStack = recipe.getOutputStack(stack);
                        }
                    }

                    // Reset item in inventory
                    inventory.setStackInSlot(i, outputStack);
                }

                field_145850_b.func_175698_g(above);
                updateBlock();
                // Since there will be no items in the pit kiln at this point
                TEPlacedItem.convertPitKilnToPlacedItem(field_145850_b, field_174879_c);
            }
        }
    }

    @Override
    public void onBreakBlock(World worldIn, BlockPos pos)
    {
        strawItems.forEach(i -> InventoryHelper.func_180173_a(field_145850_b, pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), i));
        logItems.forEach(i -> InventoryHelper.func_180173_a(field_145850_b, pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), i));
        super.onBreakBlock(worldIn, pos);
    }

    public boolean isLit()
    {
        return burnTicksToGo > 0;
    }

    public boolean hasFuel()
    {
        return !(logItems.stream().anyMatch(ItemStack::func_190926_b) || strawItems.stream().anyMatch(ItemStack::func_190926_b));
    }

    /**
     * @return true if an action was taken (passed back through onItemRightClick)
     */
    public boolean onRightClick(EntityPlayer player, ItemStack stack, boolean x, boolean z)
    {
        if (isLit())
        {
            return false;
        }

        // Try and extract an item
        if (player.func_70093_af())
        {
            // This will search through the logItems, then the strawItems
            ItemStack dropStack = logItems.stream().filter(i -> !i.func_190926_b()).findFirst().orElseGet(() -> strawItems.stream().filter(i -> !i.func_190926_b()).findFirst().orElse(ItemStack.field_190927_a));
            if (!dropStack.func_190926_b())
            {
                ItemHandlerHelper.giveItemToPlayer(player, dropStack.func_77979_a(1));
                updateBlock();

                if (getStrawCount() == 0)
                {
                    TEPlacedItem.convertPitKilnToPlacedItem(field_145850_b, field_174879_c);
                }
                return true;
            }
        }
        else if (!stack.func_190926_b())
        {
            // Insert an item
            int strawCount = getStrawCount(), logCount = getLogCount();

            // Straw
            if (OreDictionaryHelper.doesStackMatchOre(stack, "straw") && strawCount < STRAW_NEEDED)
            {
                addStraw(stack.func_77979_a(1));
                updateBlock();
                return true;
            }

            // Straw via thatch block (special exception)
            if (stack.func_77973_b() == Item.func_150898_a(BlocksTFC.THATCH) && strawCount <= STRAW_NEEDED - 4)
            {
                stack.func_190918_g(1);
                addStrawBlock();
                updateBlock();
                return true;
            }
            // Only insert logItems if all strawItems is inserted
            if (strawCount == STRAW_NEEDED)
            {
                // Logs
                if (OreDictionaryHelper.doesStackMatchOre(stack, "logWood") && logCount < WOOD_NEEDED)
                {
                    addLog(stack.func_77979_a(1));
                    updateBlock();
                    return true;
                }
                // Light
                if (logCount == WOOD_NEEDED && stack.func_77973_b() instanceof ItemFlintAndSteel)
                {
                    // Flint and steel should light immediately
                    return tryLight();
                }
            }
        }
        return false;
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt)
    {
        burnTicksToGo = nbt.func_74762_e("burnTicksToGo");
        ItemStackHelper.func_191283_b(nbt.func_74775_l("strawItems"), strawItems);
        ItemStackHelper.func_191283_b(nbt.func_74775_l("logItems"), logItems);

        super.func_145839_a(nbt);
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound nbt)
    {
        nbt.func_74772_a("burnTicksToGo", burnTicksToGo);
        nbt.func_74782_a("strawItems", ItemStackHelper.func_191282_a(new NBTTagCompound(), strawItems));
        nbt.func_74782_a("logItems", ItemStackHelper.func_191282_a(new NBTTagCompound(), logItems));
        return super.func_189515_b(nbt);
    }

    public int getLogCount()
    {
        return (int) logItems.stream().filter(i -> !i.func_190926_b()).count();
    }

    public int getStrawCount()
    {
        return (int) strawItems.stream().filter(i -> !i.func_190926_b()).count();
    }

    public boolean tryLight()
    {
        if (hasFuel() && isValid())
        {
            BlockPos above = field_174879_c.func_177982_a(0, 1, 0);
            if (Blocks.field_150480_ab.func_176196_c(field_145850_b, above))
            {
                for (EnumFacing facing : EnumFacing.Plane.HORIZONTAL)
                {
                    if (!field_145850_b.isSideSolid(field_174879_c.func_177972_a(facing), facing.func_176734_d()))
                    {
                        return false;
                    }
                }
                burnTicksToGo = ConfigTFC.GENERAL.pitKilnTime;
                updateBlock();
                field_145850_b.func_175656_a(above, Blocks.field_150480_ab.func_176223_P());
                return true;
            }
        }
        return false;
    }

    public void assertValid()
    {
        if (isLit() && !isValid() && !field_145850_b.field_72995_K)
        {
            // Stop burning, remove all straw + wood, etc.
            if (field_145850_b.func_180495_p(field_174879_c.func_177984_a()).func_185904_a() == Material.field_151581_o)
            {
                field_145850_b.func_175698_g(field_174879_c.func_177984_a());
            }
            strawItems.clear();
            logItems.clear();
            // The easiest way to do this is just to convert to a PlacedItem:
            TEPlacedItem.convertPitKilnToPlacedItem(field_145850_b, field_174879_c);
        }
    }

    private void addStrawBlock()
    {
        for (int i = 0; i < 4; i++)
        {
            addStraw(new ItemStack(ItemsTFC.STRAW));
        }
    }

    private void addStraw(ItemStack stack)
    {
        for (int i = 0; i < strawItems.size(); i++)
        {
            if (strawItems.get(i).func_190926_b())
            {
                strawItems.set(i, stack);
                return;
            }
        }
    }

    private void addLog(ItemStack stack)
    {
        for (int i = 0; i < logItems.size(); i++)
        {
            if (logItems.get(i).func_190926_b())
            {
                logItems.set(i, stack);
                return;
            }
        }
    }

    private boolean isValid()
    {
        for (EnumFacing face : EnumFacing.field_176754_o)
        {
            if (!field_145850_b.func_180495_p(field_174879_c.func_177972_a(face)).func_185915_l())
            {
                return false;
            }
        }
        return field_145850_b.func_180495_p(field_174879_c.func_177977_b()).func_185915_l();
    }
}
